@@ -185,23 +185,23 @@ gth_file_source_catalogs_get_file_data (GthFileSource *file_source,
 
 
 typedef struct {
-	GthFileSourceCatalogs *catalogs;
+	GthFileSourceCatalogs *file_souce;
 	GthFileData           *file_data;
 	char                  *attributes;
 	ReadyCallback          ready_callback;
 	gpointer               user_data;
-	GthCatalog            *catalog;
+	GFile                 *gio_file;
 } MetadataOpData;
 
 
 static void
 metadata_op_free (MetadataOpData *metadata_op)
 {
-	gth_file_source_set_active (GTH_FILE_SOURCE (metadata_op->catalogs), FALSE);
+	gth_file_source_set_active (GTH_FILE_SOURCE (metadata_op->file_souce), FALSE);
 	g_object_unref (metadata_op->file_data);
 	g_free (metadata_op->attributes);
-	g_object_unref (metadata_op->catalog);
-	g_object_unref (metadata_op->catalogs);
+	g_object_unref (metadata_op->gio_file);
+	g_object_unref (metadata_op->file_souce);
 	g_free (metadata_op);
 }
 
@@ -213,7 +213,7 @@ write_metadata_write_buffer_ready_cb (void     **buffer,
 				      gpointer   user_data)
 {
 	MetadataOpData        *metadata_op = user_data;
-	GthFileSourceCatalogs *catalogs = metadata_op->catalogs;
+	GthFileSourceCatalogs *catalogs = metadata_op->file_souce;
 
 	metadata_op->ready_callback (G_OBJECT (catalogs), error, metadata_op->user_data);
 	metadata_op_free (metadata_op);
@@ -226,45 +226,51 @@ write_metadata_load_buffer_ready_cb (void     **buffer,
 				     GError    *error,
 				     gpointer   user_data)
 {
-	MetadataOpData        *metadata_op = user_data;
-	GthFileSourceCatalogs *catalogs = metadata_op->catalogs;
-	GFile                 *gio_file;
-	void                  *catalog_buffer;
-	gsize                  catalog_size;
+	MetadataOpData *metadata_op = user_data;
+	GthCatalog     *catalog;
+	void           *catalog_buffer;
+	gsize           catalog_size;
 
 	if (error != NULL) {
-		metadata_op->ready_callback (G_OBJECT (catalogs), error, metadata_op->user_data);
+		metadata_op->ready_callback (G_OBJECT (metadata_op->file_souce), error, metadata_op->user_data);
 		metadata_op_free (metadata_op);
 		return;
 	}
 
-	gth_catalog_load_from_data (metadata_op->catalog, *buffer, count, &error);
+	catalog = gth_catalog_new_from_data (*buffer, count, &error);
+	if (catalog == NULL) {
+		metadata_op->ready_callback (G_OBJECT (metadata_op->file_souce), error, metadata_op->user_data);
+		metadata_op_free (metadata_op);
+		return;
+	}
+
+	gth_catalog_set_file (catalog, metadata_op->gio_file);
 
 	if (error != NULL) {
-		metadata_op->ready_callback (G_OBJECT (catalogs), error, metadata_op->user_data);
+		metadata_op->ready_callback (G_OBJECT (metadata_op->file_souce), error, metadata_op->user_data);
+		g_object_unref (catalog);
 		metadata_op_free (metadata_op);
 		return;
 	}
 
 	if (_g_file_attributes_matches_any (metadata_op->attributes, "sort::*"))
-		gth_catalog_set_order (metadata_op->catalog,
+		gth_catalog_set_order (catalog,
 				       g_file_info_get_attribute_string (metadata_op->file_data->info, "sort::type"),
 				       g_file_info_get_attribute_boolean (metadata_op->file_data->info, "sort::inverse"));
 
-	gth_hook_invoke ("gth-catalog-read-metadata", metadata_op->catalog, metadata_op->file_data);
+	gth_hook_invoke ("gth-catalog-read-metadata", catalog, metadata_op->file_data);
 
-	catalog_buffer = gth_catalog_to_data (metadata_op->catalog, &catalog_size);
-	gio_file = gth_catalog_file_to_gio_file (metadata_op->file_data->file);
-	_g_file_write_async (gio_file,
+	catalog_buffer = gth_catalog_to_data (catalog, &catalog_size);
+	_g_file_write_async (metadata_op->gio_file,
 			     catalog_buffer,
 			     catalog_size,
 			     TRUE,
 			     G_PRIORITY_DEFAULT,
-			     gth_file_source_get_cancellable (GTH_FILE_SOURCE (metadata_op->catalogs)),
+			     gth_file_source_get_cancellable (GTH_FILE_SOURCE (metadata_op->file_souce)),
 			     write_metadata_write_buffer_ready_cb,
 			     metadata_op);
 
-	g_object_unref (gio_file);
+	g_object_unref (catalog);
 }
 
 
@@ -278,7 +284,6 @@ gth_file_source_catalogs_write_metadata (GthFileSource *file_source,
 	GthFileSourceCatalogs *catalogs = (GthFileSourceCatalogs *) file_source;
 	char                  *uri;
 	MetadataOpData        *metadata_op;
-	GFile                 *gio_file;
 
 	uri = g_file_get_uri (file_data->file);
 	if (! g_str_has_suffix (uri, ".gqv")
@@ -291,7 +296,7 @@ gth_file_source_catalogs_write_metadata (GthFileSource *file_source,
 	}
 
 	metadata_op = g_new0 (MetadataOpData, 1);
-	metadata_op->catalogs = g_object_ref (catalogs);
+	metadata_op->file_souce = g_object_ref (catalogs);
 	metadata_op->file_data = g_object_ref (file_data);
 	metadata_op->attributes = g_strdup (attributes);
 	metadata_op->ready_callback = callback;
@@ -300,16 +305,13 @@ gth_file_source_catalogs_write_metadata (GthFileSource *file_source,
 	gth_file_source_set_active (GTH_FILE_SOURCE (catalogs), TRUE);
 	g_cancellable_reset (gth_file_source_get_cancellable (file_source));
 
-	metadata_op->catalog = gth_catalog_new ();
-	gio_file = gth_file_source_to_gio_file (file_source, file_data->file);
-	gth_catalog_set_file (metadata_op->catalog, gio_file);
-	_g_file_load_async (gio_file,
+	metadata_op->gio_file = gth_file_source_to_gio_file (file_source, file_data->file);
+	_g_file_load_async (metadata_op->gio_file,
 			    G_PRIORITY_DEFAULT,
 			    gth_file_source_get_cancellable (file_source),
 			    write_metadata_load_buffer_ready_cb,
 			    metadata_op);
 
-	g_object_unref (gio_file);
 	g_free (uri);
 }
 
@@ -343,11 +345,12 @@ read_metadata_catalog_ready_cb (GObject  *object,
 {
 	ReadMetadataOpData *read_metadata = user_data;
 
-	/* ignore errors */
-	if (error != NULL)
+	if (error != NULL) {
+		/* ignore errors */
 		g_clear_error (&error);
-
-	if (object != NULL) {
+	}
+	else {
+		g_assert (object != NULL);
 		gth_catalog_update_metadata (GTH_CATALOG (object), read_metadata->file_data);
 		g_object_unref (object);
 	}
@@ -435,6 +438,7 @@ gth_file_source_catalogs_rename (GthFileSource *file_source,
 	if (catalog != NULL) {
 		char  *uri;
 		char  *clean_name;
+		char  *ext;
 		char  *name;
 		GFile *gio_new_file;
 		char  *data;
@@ -443,7 +447,8 @@ gth_file_source_catalogs_rename (GthFileSource *file_source,
 
 		uri = g_file_get_uri (file);
 		clean_name = _g_filename_clear_for_file (edit_name);
-		name = g_strconcat (clean_name, _g_uri_get_file_extension (uri), NULL);
+		ext = _g_uri_get_extension (uri);
+		name = g_strconcat (clean_name, ext, NULL);
 		new_file = g_file_get_child_for_display_name (parent, name, &error);
 		gth_catalog_set_file (catalog, new_file);
 		gth_catalog_set_name (catalog, edit_name);
@@ -477,6 +482,7 @@ gth_file_source_catalogs_rename (GthFileSource *file_source,
 		g_free (data);
 		g_object_unref (gio_new_file);
 		g_free (clean_name);
+		g_free (ext);
 		g_free (name);
 		g_free (uri);
 	}
@@ -528,7 +534,6 @@ typedef struct {
 	ForEachChildCallback   for_each_file_func;
 	ReadyCallback          ready_func;
 	gpointer               user_data;
-	GthCatalog            *catalog;
 	GList                 *to_visit;
 } ForEachChildData;
 
@@ -537,7 +542,6 @@ static void
 for_each_child_data_free (ForEachChildData *data)
 {
 	_g_object_list_unref (data->to_visit);
-	g_object_ref (data->catalog);
 	g_free (data->attributes);
 	g_object_ref (data->file_source);
 }
@@ -674,15 +678,14 @@ for_each_child__visit_file (ForEachChildData *data,
 	    || g_str_has_suffix (uri, ".catalog")
 	    || g_str_has_suffix (uri, ".search"))
 	{
-		gth_catalog_set_file (data->catalog, gio_file);
-		gth_catalog_list_async (data->catalog,
+		gth_catalog_list_async (gio_file,
 					data->attributes,
 					gth_file_source_get_cancellable (data->file_source),
 					for_each_child__catalog_list_ready_cb,
 					data);
 	}
 	else
-		g_directory_foreach_child (gio_file,
+		_g_directory_foreach_child (gio_file,
 					   FALSE,
 					   TRUE,
 					   GFILE_STANDARD_ATTRIBUTES_WITH_FAST_CONTENT_TYPE,
@@ -743,7 +746,6 @@ gth_file_source_catalogs_for_each_child (GthFileSource        *file_source,
 	data->for_each_file_func = for_each_file_func;
 	data->ready_func = ready_func;
 	data->user_data = user_data;
-	data->catalog = gth_catalog_new ();
 
 	gio_parent = gth_file_source_to_gio_file (file_source, parent);
 	g_file_query_info_async (gio_parent,
@@ -825,6 +827,7 @@ catalog_ready_cb (GObject  *catalog,
 		return;
 	}
 
+	g_assert (catalog != NULL);
 	cod->catalog = (GthCatalog *) catalog;
 
 	if (cod->destination_position >= 0)
@@ -944,13 +947,13 @@ copy_catalog_ready_cb (GError   *error,
 	first_file = ccd->file_list->data;
 
 	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
-		char       *uri;
-		const char *extension;
-		char       *msg;
-		GtkWidget  *d;
+		char      *uri;
+		char      *extension;
+		char      *msg;
+		GtkWidget *d;
 
 		uri = g_file_get_uri (first_file);
-		extension = _g_uri_get_file_extension (uri);
+		extension = _g_uri_get_extension (uri);
 		if ((g_strcmp0 (extension, ".catalog") == 0) || (g_strcmp0 (extension, ".search") == 0))
 			msg = g_strdup_printf (_("The catalog “%s” already exists, do you want to overwrite it?"), g_file_info_get_display_name (ccd->destination->info));
 		else
@@ -972,6 +975,7 @@ copy_catalog_ready_cb (GError   *error,
 		gtk_widget_show (d);
 
 		g_free (msg);
+		g_free (extension);
 		g_free (uri);
 
 		return;
@@ -1020,19 +1024,19 @@ _gth_file_source_catalogs_copy_catalog (CopyCatalogData      *ccd,
 	gio_list = gth_file_source_to_gio_file_list (ccd->file_source, ccd->file_list);
 	gio_destination = gth_file_source_to_gio_file (ccd->file_source, ccd->destination->file);
 
-	_g_copy_files_async (gio_list,
-			     gio_destination,
-			     ccd->move,
-			     GTH_FILE_COPY_DEFAULT,
-			     default_response,
-			     G_PRIORITY_DEFAULT,
-			     gth_file_source_get_cancellable (ccd->file_source),
-			     ccd->progress_callback,
-			     ccd->user_data,
-			     ccd->dialog_callback,
-			     ccd->user_data,
-			     copy_catalog_ready_cb,
-			     ccd);
+	_g_file_list_copy_async (gio_list,
+				 gio_destination,
+				 ccd->move,
+				 GTH_FILE_COPY_DEFAULT,
+				 default_response,
+				 G_PRIORITY_DEFAULT,
+				 gth_file_source_get_cancellable (ccd->file_source),
+				 ccd->progress_callback,
+				 ccd->user_data,
+				 ccd->dialog_callback,
+				 ccd->user_data,
+				 copy_catalog_ready_cb,
+				 ccd);
 
 	g_object_unref (gio_destination);
 	_g_object_list_unref (gio_list);
@@ -1068,7 +1072,7 @@ gth_file_source_catalogs_copy (GthFileSource    *file_source,
 
 	first_file = file_list->data;
 	if (g_file_has_uri_scheme (first_file, "catalog")) {
-		if (g_strcmp0 (g_file_info_get_content_type (destination->info), "gthumb/catalog") == 0) {
+		if (g_strcmp0 (_g_file_info_get_content_type (destination->info), "gthumb/catalog") == 0) {
 			CopyCatalogData *ccd;
 			const char      *msg;
 			GtkWidget       *d;
@@ -1140,24 +1144,16 @@ gth_file_source_catalogs_copy (GthFileSource    *file_source,
 			g_free (message);
 		}
 
-		_g_query_info_async (cod->file_list,
-				     GTH_LIST_DEFAULT,
-				     GFILE_NAME_TYPE_ATTRIBUTES,
-				     gth_file_source_get_cancellable (file_source),
-				     copy__file_list_info_ready_cb,
-				     cod);
+		_g_file_list_query_info_async (cod->file_list,
+					       GTH_LIST_DEFAULT,
+					       GFILE_NAME_TYPE_ATTRIBUTES,
+					       gth_file_source_get_cancellable (file_source),
+					       copy__file_list_info_ready_cb,
+					       cod);
 	}
 }
 
 
-static gboolean
-gth_file_source_catalogs_can_cut (GthFileSource *file_source,
-				  GFile         *file)
-{
-	return g_file_has_uri_scheme (file, "catalog");
-}
-
-
 static gboolean
 gth_file_source_catalogs_is_reorderable (GthFileSource *file_source)
 {
@@ -1247,6 +1243,7 @@ reorder_catalog_ready_cb (GObject  *object,
 		return;
 	}
 
+	g_assert (object != NULL);
 	catalog = (GthCatalog *) object;
 	reorder_data->new_order = reorder_catalog_list (catalog,
 							reorder_data->visible_files,
@@ -1370,15 +1367,8 @@ catalog_buffer_ready_cb (void     **buffer,
 		return;
 	}
 
-	data->catalog = gth_hook_invoke_get ("gth-catalog-load-from-data", *buffer);
+	data->catalog = gth_catalog_new_from_data (*buffer, count, &error);
 	if (data->catalog == NULL) {
-		error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_FAILED, _("Invalid file format"));
-		remove_from_catalog_end (error, data);
-		return;
-	}
-
-	gth_catalog_load_from_data (data->catalog, *buffer, count, &error);
-	if (error != NULL) {
 		remove_from_catalog_end (error, data);
 		return;
 	}
@@ -1453,6 +1443,68 @@ gth_file_source_catalogs_deleted_from_disk (GthFileSource *file_source,
 }
 
 
+static GdkDragAction
+gth_file_source_catalogs_get_drop_actions (GthFileSource *file_source,
+					   GFile         *destination,
+					   GFile         *file)
+{
+	GdkDragAction  actions = 0;
+	char          *dest_uri;
+	char          *dest_scheme;
+	char          *dest_ext;
+	gboolean       dest_is_catalog;
+	char          *file_uri;
+	char          *file_scheme;
+	char          *file_ext;
+	gboolean       file_is_catalog;
+
+	dest_uri = g_file_get_uri (destination);
+	dest_scheme = gth_main_get_source_scheme (dest_uri);
+	dest_ext = _g_uri_get_extension (dest_uri);
+	dest_is_catalog = _g_str_equal (dest_ext, ".catalog") || _g_str_equal (dest_ext, ".search");
+
+	file_uri = g_file_get_uri (file);
+	file_scheme = gth_main_get_source_scheme (file_uri);
+	file_ext = _g_uri_get_extension (file_uri);
+	file_is_catalog = _g_str_equal (file_ext, ".catalog") || _g_str_equal (file_ext, ".search");
+
+	if (_g_str_equal (dest_scheme, "catalog")
+		&& dest_is_catalog
+		&& _g_str_equal (file_scheme, "file"))
+	{
+		/* Copy files into a catalog. */
+		actions = GDK_ACTION_COPY;
+	}
+
+	else if (_g_str_equal (file_scheme, "catalog")
+		&& file_is_catalog
+		&& _g_str_equal (dest_scheme, "catalog")
+		&& ! dest_is_catalog)
+	{
+		/* Move a catalog into a library. */
+		actions = GDK_ACTION_MOVE;
+	}
+
+	else if (_g_str_equal (file_scheme, "catalog")
+		&& ! file_is_catalog
+		&& _g_str_equal (dest_scheme, "catalog")
+		&& ! dest_is_catalog)
+	{
+		/* Move a library into another library. */
+		actions = GDK_ACTION_MOVE;
+	}
+
+	g_free (file_ext);
+	g_free (file_scheme);
+	g_free (file_uri);
+	g_free (dest_ext);
+	g_free (dest_scheme);
+	g_free (dest_uri);
+
+	return actions;
+}
+
+
 static void
 gth_file_source_catalogs_finalize (GObject *object)
 {
@@ -1485,11 +1537,11 @@ gth_file_source_catalogs_class_init (GthFileSourceCatalogsClass *class)
 	file_source_class->rename = gth_file_source_catalogs_rename;
 	file_source_class->for_each_child = gth_file_source_catalogs_for_each_child;
 	file_source_class->copy = gth_file_source_catalogs_copy;
-	file_source_class->can_cut = gth_file_source_catalogs_can_cut;
 	file_source_class->is_reorderable  = gth_file_source_catalogs_is_reorderable;
 	file_source_class->reorder = gth_file_source_catalogs_reorder;
 	file_source_class->remove = gth_file_source_catalogs_remove;
 	file_source_class->deleted_from_disk = gth_file_source_catalogs_deleted_from_disk;
+	file_source_class->get_drop_actions = gth_file_source_catalogs_get_drop_actions;
 }
 
 
