@@ -26,9 +26,6 @@
 #define GET_WIDGET(x) (_gtk_builder_get_widget (self->priv->builder, (x)))
 
 
-G_DEFINE_TYPE (GthContactSheetThemeDialog, gth_contact_sheet_theme_dialog, GTK_TYPE_DIALOG)
-
-
 struct _GthContactSheetThemeDialogPrivate {
 	GtkBuilder           *builder;
 	GtkWidget            *copy_from_button;
@@ -38,6 +35,12 @@ struct _GthContactSheetThemeDialogPrivate {
 };
 
 
+G_DEFINE_TYPE_WITH_CODE (GthContactSheetThemeDialog,
+			 gth_contact_sheet_theme_dialog,
+			 GTK_TYPE_DIALOG,
+			 G_ADD_PRIVATE (GthContactSheetThemeDialog))
+
+
 static void
 gth_contact_sheet_theme_dialog_finalize (GObject *object)
 {
@@ -58,8 +61,6 @@ gth_contact_sheet_theme_dialog_class_init (GthContactSheetThemeDialogClass *klas
 {
 	GObjectClass *object_class;
 
-	g_type_class_add_private (klass, sizeof (GthContactSheetThemeDialogPrivate));
-
 	object_class = (GObjectClass*) klass;
 	object_class->finalize = gth_contact_sheet_theme_dialog_finalize;
 }
@@ -125,17 +126,17 @@ update_theme_from_controls (GthContactSheetThemeDialog *self)
 
 	/* header */
 
-	self->priv->theme->header_font_name = g_strdup (gtk_font_button_get_font_name (GTK_FONT_BUTTON (GET_WIDGET ("header_fontpicker"))));
+	self->priv->theme->header_font_name = g_strdup (gtk_font_chooser_get_font (GTK_FONT_CHOOSER (GET_WIDGET ("header_fontpicker"))));
 	gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (GET_WIDGET ("header_colorpicker")), &self->priv->theme->header_color);
 
 	/* footer */
 
-	self->priv->theme->footer_font_name = g_strdup (gtk_font_button_get_font_name (GTK_FONT_BUTTON (GET_WIDGET ("footer_fontpicker"))));
+	self->priv->theme->footer_font_name = g_strdup (gtk_font_chooser_get_font (GTK_FONT_CHOOSER (GET_WIDGET ("footer_fontpicker"))));
 	gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (GET_WIDGET ("footer_colorpicker")), &self->priv->theme->footer_color);
 
 	/* caption */
 
-	self->priv->theme->caption_font_name = g_strdup (gtk_font_button_get_font_name (GTK_FONT_BUTTON (GET_WIDGET ("caption_fontpicker"))));
+	self->priv->theme->caption_font_name = g_strdup (gtk_font_chooser_get_font (GTK_FONT_CHOOSER (GET_WIDGET ("caption_fontpicker"))));
 	gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (GET_WIDGET ("caption_colorpicker")), &self->priv->theme->caption_color);
 }
 
@@ -185,7 +186,7 @@ gth_contact_sheet_theme_dialog_init (GthContactSheetThemeDialog *self)
 {
 	GtkWidget *content;
 
-	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_CONTACT_SHEET_THEME_DIALOG, GthContactSheetThemeDialogPrivate);
+	self->priv = gth_contact_sheet_theme_dialog_get_instance_private (self);
 	self->priv->builder = _gtk_builder_new_from_file ("contact-sheet-theme-properties.ui", "contact_sheet");
 	self->priv->theme = NULL;
 	self->priv->all_themes = NULL;
@@ -375,13 +376,13 @@ update_controls_from_theme (GthContactSheetThemeDialog *self,
 	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("frame_style_combobox")), theme->frame_style);
 	gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (GET_WIDGET ("frame_colorpicker")), &theme->frame_color);
 
-	gtk_font_button_set_font_name (GTK_FONT_BUTTON (GET_WIDGET ("header_fontpicker")), theme->header_font_name);
+	gtk_font_chooser_set_font (GTK_FONT_CHOOSER (GET_WIDGET ("header_fontpicker")), theme->header_font_name);
 	gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (GET_WIDGET ("header_colorpicker")), &theme->header_color);
 
-	gtk_font_button_set_font_name (GTK_FONT_BUTTON (GET_WIDGET ("footer_fontpicker")), theme->footer_font_name);
+	gtk_font_chooser_set_font (GTK_FONT_CHOOSER (GET_WIDGET ("footer_fontpicker")), theme->footer_font_name);
 	gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (GET_WIDGET ("footer_colorpicker")), &theme->footer_color);
 
-	gtk_font_button_set_font_name (GTK_FONT_BUTTON (GET_WIDGET ("caption_fontpicker")), theme->caption_font_name);
+	gtk_font_chooser_set_font (GTK_FONT_CHOOSER (GET_WIDGET ("caption_fontpicker")), theme->caption_font_name);
 	gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (GET_WIDGET ("caption_colorpicker")), &theme->caption_color);
 
 	update_preview (self);
