@@ -1456,7 +1456,8 @@ exiv2_supports_writes (const char *mime_type)
 {
 	return (g_content_type_equals (mime_type, "image/jpeg")
 		|| g_content_type_equals (mime_type, "image/tiff")
-		|| g_content_type_equals (mime_type, "image/png"));
+		|| g_content_type_equals (mime_type, "image/png")
+		|| g_content_type_equals (mime_type, "image/webp"));
 }
 
 
