@@ -370,14 +370,15 @@ clipboard_received_cb (GtkClipboard     *clipboard,
 		       GtkSelectionData *selection_data,
 		       gpointer          user_data)
 {
-	PasteData   *paste_data = user_data;
-	GthBrowser  *browser = paste_data->browser;
-	const char  *raw_data;
-	char       **clipboard_data;
-	int          i;
-	GtkTreePath *path;
-	int          position;
-	GthTask     *task;
+	PasteData      *paste_data = user_data;
+	GthBrowser     *browser = paste_data->browser;
+	const char     *raw_data;
+	char          **clipboard_data;
+	int             i;
+	GdkDragAction   actions;
+	GtkTreePath    *path;
+	int             position;
+	GthTask        *task;
 
 	raw_data = (const char *) gtk_selection_data_get_data (selection_data);
 	if (raw_data == NULL) {
@@ -400,7 +401,19 @@ clipboard_received_cb (GtkClipboard     *clipboard,
 	paste_data->files = g_list_reverse (paste_data->files);
 	paste_data->file_source = gth_main_get_file_source (paste_data->destination->file);
 
-	if (paste_data->cut && ! gth_file_source_can_cut (paste_data->file_source, paste_data->files->data)) {
+	actions = gth_file_source_get_drop_actions (paste_data->file_source,
+						    paste_data->destination->file,
+						    G_FILE (paste_data->files->data));
+	if (actions == 0) {
+		_gtk_error_dialog_run (GTK_WINDOW (browser),
+				       "%s",
+				       _("Could not perform the operation"));
+		g_strfreev (clipboard_data);
+		paste_data_free (paste_data);
+		return;
+	}
+
+	if (paste_data->cut && ((actions & GDK_ACTION_MOVE) == 0)) {
 		GtkWidget *dialog;
 		int        response;
 
@@ -416,6 +429,7 @@ clipboard_received_cb (GtkClipboard     *clipboard,
 		gtk_widget_destroy (dialog);
 
 		if (response == GTK_RESPONSE_CANCEL) {
+			g_strfreev (clipboard_data);
 			paste_data_free (paste_data);
 			return;
 		}
@@ -442,6 +456,7 @@ clipboard_received_cb (GtkClipboard     *clipboard,
 	gth_browser_exec_task (browser, task, GTH_TASK_FLAGS_DEFAULT);
 
 	g_object_unref (task);
+	g_strfreev (clipboard_data);
 	paste_data_free (paste_data);
 }
 
@@ -528,6 +543,70 @@ gth_browser_activate_delete (GSimpleAction *action,
 }
 
 
+static void
+remove_from_source (GthBrowser *browser,
+		    gboolean    permanently)
+{
+	GthFileSource *source;
+	GthFileData   *location;
+	GList         *items;
+	GList         *file_data_list;
+
+	if (permanently) {
+		/* Use the VFS file source to delete the files from the
+		 * disk. */
+
+		source = gth_main_get_file_source_for_uri ("file:///");
+		location = NULL;
+	}
+	else {
+		/* Removes the files from the current location,
+		 * for example: when viewing a catalog removes
+		 * the files from the catalog; when viewing a
+		 * folder removes the files from the folder. */
+
+		source = _g_object_ref (gth_browser_get_location_source (browser));
+		location = gth_browser_get_location_data (browser);
+	}
+
+	if (source == NULL)
+		return;
+
+	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
+	if (items == NULL)
+		return;
+
+	file_data_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
+	gth_file_source_remove (source,
+				location,
+				file_data_list,
+				permanently,
+				GTK_WINDOW (browser));
+
+	_g_object_list_unref (file_data_list);
+	_gtk_tree_path_list_free (items);
+	_g_object_unref (source);
+}
+
+
+void
+gth_browser_activate_remove_from_source (GSimpleAction *action,
+					 GVariant      *parameter,
+					 gpointer       user_data)
+{
+	remove_from_source (GTH_BROWSER (user_data), FALSE);
+}
+
+
+void
+gth_browser_activate_remove_from_source_permanently (GSimpleAction *action,
+						     GVariant      *parameter,
+						     gpointer       user_data)
+{
+	remove_from_source (GTH_BROWSER (user_data), TRUE);
+}
+
+
 void
 gth_browser_activate_rename (GSimpleAction *action,
 			     GVariant      *parameter,
@@ -669,7 +748,7 @@ copy_to_folder_dialog (GthBrowser *browser,
 	start_uri = g_settings_get_string (settings, PREF_FILE_MANAGER_COPY_LAST_FOLDER);
 	if ((start_uri == NULL) || (strcmp (start_uri, "") == 0)) {
 		g_free (start_uri);
-		start_uri = g_strdup (get_home_uri ());
+		start_uri = g_strdup (_g_uri_get_home ());
 	}
 	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (dialog), start_uri);
 	g_free(start_uri);
@@ -978,3 +1057,27 @@ gth_browser_activate_folder_context_move_to (GSimpleAction *action,
 {
 	copy_folder_to_folder (GTH_BROWSER (user_data), TRUE);
 }
+
+
+void
+gth_browser_activate_open_with_gimp (GSimpleAction *action,
+				     GVariant      *parameter,
+				     gpointer       user_data)
+{
+	GthBrowser *browser = user_data;
+	GList      *items;
+	GList      *file_data_list;
+	GList      *file_list;
+
+	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
+	if (items == NULL)
+		return;
+
+	file_data_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
+	file_list = gth_file_data_list_to_file_list (file_data_list);
+	_g_launch_command (GTK_WIDGET (browser), "gimp %U", "Gimp", G_APP_INFO_CREATE_SUPPORTS_URIS, file_list);
+
+	_g_object_list_unref (file_list);
+	_g_object_list_unref (file_data_list);
+	_gtk_tree_path_list_free (items);
+}
