@@ -181,7 +181,7 @@ void
 flickr_photo_set_id (FlickrPhoto *self,
 		     const char  *value)
 {
-	_g_strset (&self->id, value);
+	_g_str_set (&self->id, value);
 }
 
 
@@ -189,7 +189,7 @@ void
 flickr_photo_set_secret (FlickrPhoto *self,
 			 const char  *value)
 {
-	_g_strset (&self->secret, value);
+	_g_str_set (&self->secret, value);
 }
 
 
@@ -197,7 +197,7 @@ void
 flickr_photo_set_server (FlickrPhoto *self,
 			 const char  *value)
 {
-	_g_strset (&self->server, value);
+	_g_str_set (&self->server, value);
 }
 
 
@@ -205,7 +205,7 @@ void
 flickr_photo_set_farm (FlickrPhoto *self,
 		       const char  *value)
 {
-	_g_strset (&self->farm, value);
+	_g_str_set (&self->farm, value);
 }
 
 
@@ -213,7 +213,7 @@ void
 flickr_photo_set_title (FlickrPhoto *self,
 			const char  *value)
 {
-	_g_strset (&self->title, value);
+	_g_str_set (&self->title, value);
 }
 
 
@@ -273,7 +273,7 @@ flickr_photo_set_url (FlickrPhoto *self,
 		      FlickrUrl    size,
 		      const char  *value)
 {
-	_g_strset (&(self->url[size]), value);
+	_g_str_set (&(self->url[size]), value);
 	if (self->url[size] == NULL)
 		self->url[size] = flickr_get_static_url (self, size);
 
@@ -281,7 +281,7 @@ flickr_photo_set_url (FlickrPhoto *self,
 		int other_size;
 		for (other_size = FLICKR_URL_O - 1; other_size >= 0; other_size--) {
 			if (self->url[other_size] != NULL) {
-				_g_strset (&(self->url[size]), self->url[other_size]);
+				_g_str_set (&(self->url[size]), self->url[other_size]);
 				break;
 			}
 		}
@@ -293,7 +293,7 @@ void
 flickr_photo_set_original_format (FlickrPhoto *self,
 				  const char  *value)
 {
-	_g_strset (&self->original_format, value);
+	_g_str_set (&self->original_format, value);
 
 	g_free (self->mime_type);
 	self->mime_type = NULL;
@@ -306,5 +306,5 @@ void
 flickr_photo_set_original_secret (FlickrPhoto *self,
 				  const char  *value)
 {
-	_g_strset (&self->original_secret, value);
+	_g_str_set (&self->original_secret, value);
 }
