@@ -24,10 +24,14 @@
 #include <glib/gi18n.h>
 #include <glib-object.h>
 #include <gthumb.h>
+#include "actions.h"
 #include "callbacks.h"
 #include "shortcuts.h"
 
 
+#define BROWSER_DATA_KEY "image-viewer-browser-data"
+
+
 static const GthShortcut shortcuts[] = {
 	{ "image-zoom-in", N_("Zoom in"), GTH_SHORTCUT_CONTEXT_VIEWER, GTH_SHORTCUT_CATEGORY_IMAGE_VIEWER, "plus" },
 	{ "image-zoom-out", N_("Zoom out"), GTH_SHORTCUT_CONTEXT_VIEWER, GTH_SHORTCUT_CATEGORY_IMAGE_VIEWER, "minus" },
@@ -56,9 +60,30 @@ static const GthShortcut shortcuts[] = {
 	{ "scroll-page-down", N_("Scroll down fast"), GTH_SHORTCUT_CONTEXT_VIEWER, GTH_SHORTCUT_CATEGORY_SCROLL_IMAGE, "<Shift>Down" },
 
 	{ "scroll-to-center", N_("Scroll to center"), GTH_SHORTCUT_CONTEXT_VIEWER, GTH_SHORTCUT_CATEGORY_SCROLL_IMAGE, "<Alt>Down" },
+
+	{ "apply-editor-changes", NULL, GTH_SHORTCUT_CONTEXT_EDITOR, GTH_SHORTCUT_CATEGORY_HIDDEN, "Return" },
 };
 
 
+static void
+update_open_clipboard_sensitivity (GthBrowser   *browser,
+				   GtkClipboard *clipboard)
+{
+	if (clipboard == NULL)
+		clipboard = gtk_widget_get_clipboard (GTK_WIDGET (browser), GDK_SELECTION_CLIPBOARD);
+	gth_window_enable_action (GTH_WINDOW (browser), "open-clipboard", gtk_clipboard_wait_is_image_available (clipboard));
+}
+
+
+static void
+clipboard_owner_change_cb (GtkClipboard *clipboard,
+			   GdkEvent     *event,
+			   gpointer      user_data)
+{
+	update_open_clipboard_sensitivity (GTH_BROWSER (user_data), clipboard);
+}
+
+
 void
 image_viewer__gth_browser_construct_cb (GthBrowser *browser)
 {
@@ -68,4 +93,25 @@ image_viewer__gth_browser_construct_cb (GthBrowser *browser)
 					 GTH_SHORTCUT_VIEWER_CONTEXT_IMAGE,
 					 shortcuts,
 					 G_N_ELEMENTS (shortcuts));
+
+	GtkClipboard *clipboard = gtk_widget_get_clipboard (GTK_WIDGET (browser), GDK_SELECTION_CLIPBOARD);
+	g_signal_connect (clipboard,
+		"owner_change",
+		G_CALLBACK (clipboard_owner_change_cb),
+		browser);
+
+	update_open_clipboard_sensitivity (browser, clipboard);
+}
+
+
+void
+image_viewer__gth_browser_close_cb (GthBrowser *browser)
+{
+	g_return_if_fail (GTH_IS_BROWSER (browser));
+
+	GtkClipboard *clipboard = gtk_widget_get_clipboard (GTK_WIDGET (browser), GDK_SELECTION_CLIPBOARD);
+	g_signal_handlers_disconnect_by_func (
+		clipboard,
+		G_CALLBACK (clipboard_owner_change_cb),
+		browser);
 }
