@@ -24,8 +24,8 @@
 
 #include <gthumb.h>
 
-void      list_tools__gth_browser_construct_cb           (GthBrowser  *browser);
-gpointer  list_tools__gth_browser_file_list_key_press_cb (GthBrowser  *browser,
-							  GdkEventKey *event);
+void list_tools__gth_browser_construct_cb                (GthBrowser *browser);
+void list_tools__gth_browser_selection_changed_cb        (GthBrowser *browser,
+							  int         n_selected);
 
 #endif /* CALLBACKS_H */
