@@ -26,6 +26,7 @@
 #include "glib-utils.h"
 #include "gth-image-selector.h"
 #include "gth-marshal.h"
+#include "gtk-utils.h"
 
 
 #define DEFAULT_BORDER 40
@@ -44,8 +45,9 @@ typedef struct {
 
 
 static EventArea *
-event_area_new (int           id,
-		GdkCursorType cursor_type)
+event_area_new (GtkWidget        *widget,
+		int               id,
+		GdkCursorType     cursor_type)
 {
 	EventArea *event_area;
 
@@ -57,7 +59,7 @@ event_area_new (int           id,
 	event_area->area.y = 0;
 	event_area->area.width = 0;
 	event_area->area.height = 0;
-	event_area->cursor = gdk_cursor_new_for_display (gdk_display_get_default (), cursor_type);
+	event_area->cursor = _gdk_cursor_new_for_widget (widget, cursor_type);
 
 	return event_area;
 }
@@ -227,8 +229,9 @@ static void gth_image_selector_gth_image_tool_interface_init (GthImageViewerTool
 G_DEFINE_TYPE_WITH_CODE (GthImageSelector,
 			 gth_image_selector,
 			 G_TYPE_OBJECT,
+			 G_ADD_PRIVATE (GthImageSelector)
 			 G_IMPLEMENT_INTERFACE (GTH_TYPE_IMAGE_VIEWER_TOOL,
-					 	gth_image_selector_gth_image_tool_interface_init))
+						gth_image_selector_gth_image_tool_interface_init))
 
 
 static gboolean
@@ -292,7 +295,7 @@ add_event_area (GthImageSelector *self,
 {
 	EventArea *event_area;
 
-	event_area = event_area_new (area_id, cursor_type);
+	event_area = event_area_new (GTK_WIDGET (self->priv->viewer), area_id, cursor_type);
 	self->priv->event_list = g_list_prepend (self->priv->event_list, event_area);
 }
 
@@ -636,9 +639,9 @@ gth_image_selector_realize (GthImageViewerTool *base)
 	GthImageSelector *self = GTH_IMAGE_SELECTOR (base);
 
 	if (self->priv->type == GTH_SELECTOR_TYPE_REGION)
-		self->priv->default_cursor = gdk_cursor_new_for_display (gdk_display_get_default (), GDK_CROSSHAIR /*GDK_LEFT_PTR*/);
+		self->priv->default_cursor = _gdk_cursor_new_for_widget (GTK_WIDGET (self->priv->viewer), GDK_CROSSHAIR /*GDK_LEFT_PTR*/);
 	else if (self->priv->type == GTH_SELECTOR_TYPE_POINT)
-		self->priv->default_cursor = gdk_cursor_new_for_display (gdk_display_get_default (), GDK_CROSSHAIR);
+		self->priv->default_cursor = _gdk_cursor_new_for_widget (GTK_WIDGET (self->priv->viewer), GDK_CROSSHAIR);
 	gth_image_viewer_set_cursor (self->priv->viewer, self->priv->default_cursor);
 
 	add_event_area (self, C_SELECTION_AREA, GDK_FLEUR);
@@ -1477,18 +1480,17 @@ gth_image_selector_motion_notify (GthImageViewerTool *base,
 		|| (abs (self->priv->viewer->drag_y - self->priv->viewer->drag_y_prev) > DRAG_THRESHOLD))
 	    && (self->priv->current_area != NULL))
 	{
-		GdkGrabStatus retval;
+		GdkGrabStatus status;
 
-		retval = gdk_device_grab (event->device,
-					  gtk_widget_get_window (widget),
-					  GDK_OWNERSHIP_WINDOW,
-					  FALSE,
-					  (GDK_POINTER_MOTION_MASK
-					   | GDK_POINTER_MOTION_HINT_MASK
-					   | GDK_BUTTON_RELEASE_MASK),
-					  self->priv->current_area->cursor,
-					  event->time);
-		if (retval == GDK_GRAB_SUCCESS)
+		status = gdk_seat_grab (gdk_device_get_seat (gdk_event_get_device ((GdkEvent *) event)),
+					gtk_widget_get_window (widget),
+					GDK_SEAT_CAPABILITY_ALL_POINTING,
+					TRUE,
+					self->priv->current_area->cursor,
+					(GdkEvent *) event,
+					NULL,
+					NULL);
+		if (status == GDK_GRAB_SUCCESS)
 			self->priv->viewer->dragging = TRUE;
 
 		return FALSE;
@@ -1562,6 +1564,16 @@ selection_is_valid (GthImageSelector *self)
 }
 
 
+static void
+update_selection (GthImageSelector *self)
+{
+	cairo_rectangle_int_t selection;
+
+	gth_image_selector_get_selection (self, &selection);
+	set_selection (self, selection, TRUE);
+}
+
+
 static void
 gth_image_selector_image_changed (GthImageViewerTool *base)
 {
@@ -1587,24 +1599,22 @@ gth_image_selector_image_changed (GthImageViewerTool *base)
 
 	if (! selection_is_valid (self))
 		init_selection (self);
+	else
+		update_selection (self);
 }
 
 
 static void
 gth_image_selector_zoom_changed (GthImageViewerTool *base)
 {
-	GthImageSelector      *self = GTH_IMAGE_SELECTOR (base);
-	cairo_rectangle_int_t  selection;
-
-	gth_image_selector_get_selection (self, &selection);
-	set_selection (self, selection, TRUE);
+	update_selection (GTH_IMAGE_SELECTOR (base));
 }
 
 
 static void
 gth_image_selector_init (GthImageSelector *self)
 {
-	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_IMAGE_SELECTOR, GthImageSelectorPrivate);
+	self->priv = gth_image_selector_get_instance_private (self);
 
 	self->priv->type = GTH_SELECTOR_TYPE_REGION;
 	self->priv->ratio = 1.0;
@@ -1636,8 +1646,6 @@ gth_image_selector_class_init (GthImageSelectorClass *class)
 {
 	GObjectClass *gobject_class;
 
-	g_type_class_add_private (class, sizeof (GthImageSelectorPrivate));
-
 	gobject_class = (GObjectClass*) class;
 	gobject_class->finalize = gth_image_selector_finalize;
 
