@@ -38,10 +38,6 @@
 #include "pixbuf-io.h"
 #include "typedefs.h"
 
-#ifdef GDK_WINDOWING_X11
-#include <gdk/gdkx.h>
-#endif
-
 
 static GMutex register_mutex;
 
@@ -331,17 +327,16 @@ gth_main_get_all_entry_points (void)
 }
 
 
-GFile *
+GthFileData *
 gth_main_get_nearest_entry_point (GFile *file)
 {
 	GList *list;
 	GList *scan;
 	GList *entries;
-	char  *nearest_uri;
+	GthFileData *nearest;
 	char  *uri;
 	int    file_uri_len;
 	int    min_diff;
-	GFile *nearest;
 
 	entries = NULL;
 	list = gth_main_get_all_entry_points ();
@@ -349,32 +344,34 @@ gth_main_get_nearest_entry_point (GFile *file)
 		GthFileData *entry_point = scan->data;
 
 		if (_g_file_is_parent (entry_point->file, file))
-			entries = g_list_prepend (entries, g_file_get_uri (entry_point->file));
+			entries = g_list_prepend (entries, g_object_ref (entry_point));
 	}
 
-	nearest_uri = NULL;
+	nearest = NULL;
 	uri = g_file_get_uri (file);
 	file_uri_len = strlen (uri);
 	min_diff = 0;
 	for (scan = entries; scan; scan = scan->next) {
-		char *entry_uri = scan->data;
+		GthFileData *entry_point = scan->data;
+		char *entry_uri;
 		int   entry_len;
 		int   diff;
 
+		entry_uri = g_file_get_uri (entry_point->file);
 		entry_len = strlen (entry_uri);
 		diff = abs (entry_len - file_uri_len);
 		if ((scan == entries) || (diff < min_diff)) {
 			min_diff = diff;
-			nearest_uri = entry_uri;
+			nearest = entry_point;
 		}
+
+		g_free (entry_uri);
 	}
 	g_free (uri);
 
-	nearest = NULL;
-	if (nearest_uri != NULL)
-		nearest = g_file_new_for_uri (nearest_uri);
+	_g_object_ref (nearest);
 
-	_g_string_list_free (entries);
+	_g_object_list_unref (entries);
 	_g_object_list_unref (list);
 
 	return nearest;
@@ -569,7 +566,7 @@ gth_main_get_metadata_reader (const char *id,
 	for (scan = Main->priv->metadata_provider; scan; scan = scan->next) {
 		GthMetadataProvider *registered_metadata = scan->data;
 
-		if (gth_metadata_provider_can_read (registered_metadata, mime_type, (char **)attribute_v)) {
+		if (gth_metadata_provider_can_read (registered_metadata, NULL, mime_type, (char **)attribute_v)) {
 			metadata = g_object_new (G_OBJECT_TYPE (registered_metadata), NULL);
 			break;
 		}
@@ -859,26 +856,40 @@ gth_main_get_image_loader_func (const char     *mime_type,
 				GthImageFormat  preferred_format)
 {
 	GthImageLoaderFunc  loader;
+	char               *generic_type;
 	char               *key;
-	int                 format;
+
+	generic_type = _g_mime_type_get_generic_type (mime_type);
 
 	/* give priority to the preferred format */
 
 	key = g_strdup_printf ("%s-%d", mime_type, preferred_format);
 	loader = g_hash_table_lookup (Main->priv->image_loaders, key);
+	if (loader == NULL) {
+		g_free (key);
+		key = g_strdup_printf ("%s-%d", generic_type, preferred_format);
+		loader = g_hash_table_lookup (Main->priv->image_loaders, key);
+	}
 
 	/* if the preferred format is not available, search another
 	 * format. */
 
-	for (format = 0; (loader == NULL) && (format < GTH_IMAGE_N_FORMATS); format++) {
+	for (int format = 0; (loader == NULL) && (format < GTH_IMAGE_N_FORMATS); format++) {
 		if (format == preferred_format)
 			continue;
 		g_free (key);
 		key = g_strdup_printf ("%s-%d", mime_type, format);
 		loader = g_hash_table_lookup (Main->priv->image_loaders, key);
+
+		if (loader == NULL) {
+			g_free (key);
+			key = g_strdup_printf ("%s-%d", generic_type, format);
+			loader = g_hash_table_lookup (Main->priv->image_loaders, key);
+		}
 	}
 
 	g_free (key);
+	g_free (generic_type);
 
 	return loader;
 }
@@ -1506,7 +1517,7 @@ attribute_list_reload_required (const char *old_attributes,
 
 				attr_v[0] = new_attributes_v[j];
 				attr_v[1] = NULL;
-				if (gth_metadata_provider_can_read (provider, "*", attr_v)) {
+				if (gth_metadata_provider_can_read (provider, NULL, "*", attr_v)) {
 					g_free (new_attributes_v[j]);
 					new_attributes_v[j] = NULL;
 				}
