@@ -44,8 +44,6 @@
 #include "pixbuf-utils.h"
 #include "typedefs.h"
 
-#define THUMBNAIL_LARGE_SIZE	  256
-#define THUMBNAIL_NORMAL_SIZE	  128
 #define THUMBNAIL_DIR_PERMISSIONS 0700
 #define MAX_THUMBNAILER_LIFETIME  4000   /* kill the thumbnailer after this amount of time*/
 #define CHECK_CANCELLABLE_DELAY   200
@@ -121,6 +119,28 @@ gth_thumb_loader_init (GthThumbLoader *self)
 }
 
 
+typedef struct {
+	guint event_id;
+	GCancellable *cancellable;
+	GthFileData *file_data;
+	gboolean timed_out;
+} InternalThumbnailer;
+
+
+static gboolean
+cancel_internal_thumbnailer_cb (gpointer user_data)
+{
+	InternalThumbnailer *data = user_data;
+	if (data->cancellable != NULL) {
+		g_warning ("THUMBNAILER TIMEOUT for %s\n", g_file_get_uri (data->file_data->file));
+		g_cancellable_cancel (data->cancellable);
+	}
+	data->event_id = 0;
+	data->timed_out = TRUE;
+	return G_SOURCE_REMOVE;
+}
+
+
 static GthImage *
 generate_thumbnail (GInputStream  *istream,
 		    GthFileData   *file_data,
@@ -189,7 +209,14 @@ generate_thumbnail (GInputStream  *istream,
 		 * priority to the internal loaders. */
 
 		thumbnailer = gth_main_get_image_loader_func (mime_type, GTH_IMAGE_FORMAT_CAIRO_SURFACE);
-		if (thumbnailer != NULL)
+		if (thumbnailer != NULL) {
+			InternalThumbnailer internal_thumbnailer;
+			internal_thumbnailer.file_data = file_data;
+			internal_thumbnailer.cancellable = cancellable;
+			internal_thumbnailer.timed_out = FALSE;
+			internal_thumbnailer.event_id = g_timeout_add (MAX_THUMBNAILER_LIFETIME,
+								       cancel_internal_thumbnailer_cb,
+								       &internal_thumbnailer);
 			image = thumbnailer (istream,
 					     file_data,
 					     self->priv->cache_max_size,
@@ -199,10 +226,18 @@ generate_thumbnail (GInputStream  *istream,
 					     NULL,
 					     cancellable,
 					     error);
+
+			if (internal_thumbnailer.timed_out && (cancellable != NULL))
+				g_cancellable_reset (cancellable);
+			if (internal_thumbnailer.event_id != 0)
+				g_source_remove (internal_thumbnailer.event_id);
+		}
 	}
 
-	if ((image == NULL) && (error != NULL))
-		*error = g_error_new_literal (GTH_ERROR, 0, "Could not generate the thumbnail");
+	if (((image == NULL) || gth_image_get_is_null (image)) && (error != NULL)) {
+		g_clear_error (error);
+		g_set_error_literal (error, GTH_ERROR, 0, "Could not generate the thumbnail.");
+	}
 
 	g_free (uri);
 
@@ -296,14 +331,8 @@ gth_thumb_loader_set_requested_size (GthThumbLoader *self,
 	GnomeDesktopThumbnailSize thumb_size;
 
 	self->priv->requested_size = size;
-	if (self->priv->requested_size <= THUMBNAIL_NORMAL_SIZE) {
-		self->priv->cache_max_size = THUMBNAIL_NORMAL_SIZE;
-		thumb_size = GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL;
-	}
-	else {
-		self->priv->cache_max_size = THUMBNAIL_LARGE_SIZE;
-		thumb_size = GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE;
-	}
+	thumb_size = gnome_desktop_thumbnail_size_for_size (size);
+	self->priv->cache_max_size = gnome_desktop_thumbnail_size_to_size (thumb_size);
 
 	if ((self->priv->thumb_size != thumb_size) || (self->priv->thumb_factory == NULL)) {
 		self->priv->thumb_size = thumb_size;
@@ -348,6 +377,7 @@ gth_thumb_loader_set_max_file_size (GthThumbLoader *self,
 
 
 typedef struct {
+	int                 ref;
 	GthThumbLoader     *thumb_loader;
 	GthFileData        *file_data;
 	int                 requested_size;
@@ -369,6 +399,7 @@ load_data_new (GthFileData *file_data,
 	LoadData *load_data;
 
 	load_data = g_new0 (LoadData, 1);
+	load_data->ref = 1;
 	load_data->file_data = g_object_ref (file_data);
 	load_data->requested_size = requested_size;
 
@@ -376,9 +407,24 @@ load_data_new (GthFileData *file_data,
 }
 
 
+static LoadData *
+load_data_ref (LoadData *load_data)
+{
+	g_return_val_if_fail (load_data != NULL, NULL);
+	load_data->ref++;
+	return load_data;
+}
+
+
 static void
 load_data_unref (LoadData *load_data)
 {
+	g_return_if_fail (load_data != NULL);
+
+	load_data->ref--;
+	if (load_data->ref > 0)
+		return;
+
 	g_object_unref (load_data->thumb_loader);
 	g_object_unref (load_data->file_data);
 	_g_object_unref (load_data->task);
@@ -508,9 +554,10 @@ cache_image_ready_cb (GObject      *source_object,
 				    self->priv->requested_size,
 				    self->priv->cache_max_size);
 	if (modified) {
-		cairo_surface_t *tmp = surface;
-		surface = _cairo_image_surface_scale_for_thumbnail (tmp, width, height);
-		cairo_surface_destroy (tmp);
+		cairo_surface_t *tmp;
+		tmp = _cairo_image_surface_scale_for_thumbnail (surface, width, height);
+		cairo_surface_destroy (surface);
+		surface = tmp;
 	}
 
 	load_result = g_new0 (LoadResult, 1);
@@ -586,6 +633,7 @@ _gth_thumb_loader_save_to_cache (GthThumbLoader  *self,
 							gth_file_data_get_mtime (file_data));
 
 	g_object_unref (pixbuf);
+	g_free (uri);
 
 	return TRUE;
 }
@@ -622,9 +670,10 @@ original_image_loaded_correctly (GthThumbLoader *self,
 						self->priv->cache_max_size,
 						FALSE);
 		if (modified) {
-			cairo_surface_t *tmp = local_image;
-			local_image = _cairo_image_surface_scale_for_thumbnail (tmp, width, height);
-			cairo_surface_destroy (tmp);
+			cairo_surface_t *tmp;
+			tmp = _cairo_image_surface_scale_for_thumbnail (local_image, width, height);
+			cairo_surface_destroy (local_image);
+			local_image = tmp;
 		}
 
 		_gth_thumb_loader_save_to_cache (self,
@@ -641,9 +690,10 @@ original_image_loaded_correctly (GthThumbLoader *self,
 				    self->priv->requested_size,
 				    self->priv->cache_max_size);
 	if (modified) {
-		cairo_surface_t *tmp = local_image;
-		local_image = _cairo_image_surface_scale_for_thumbnail (tmp, width, height);
-		cairo_surface_destroy (tmp);
+		cairo_surface_t *tmp;
+		tmp = _cairo_image_surface_scale_for_thumbnail (local_image, width, height);
+		cairo_surface_destroy (local_image);
+		local_image = tmp;
 	}
 
 	load_result = g_new0 (LoadResult, 1);
@@ -662,9 +712,10 @@ failed_to_load_original_image (GthThumbLoader *self,
 	char *uri;
 
 	uri = g_file_get_uri (load_data->file_data->file);
-	gnome_desktop_thumbnail_factory_create_failed_thumbnail (self->priv->thumb_factory,
-								 uri,
-								 gth_file_data_get_mtime (load_data->file_data));
+	gnome_desktop_thumbnail_factory_create_failed_thumbnail (
+		self->priv->thumb_factory,
+		uri,
+		gth_file_data_get_mtime (load_data->file_data));
 	g_task_return_error (load_data->task, g_error_new_literal (GTH_ERROR, 0, "failed to generate the thumbnail"));
 
 	g_free (uri);
@@ -715,7 +766,6 @@ watch_thumbnailer_cb (GPid     pid,
 {
 	LoadData       *load_data = user_data;
 	GthThumbLoader *self = load_data->thumb_loader;
-	GdkPixbuf      *pixbuf;
 
 	if (load_data->thumbnailer_timeout != 0) {
 		g_source_remove (load_data->thumbnailer_timeout);
@@ -737,30 +787,28 @@ watch_thumbnailer_cb (GPid     pid,
 			g_free (load_data->thumbnailer_tmpfile);
 			load_data->thumbnailer_tmpfile = NULL;
 		}
-
 		g_task_return_error (load_data->task, g_error_new_literal (G_IO_ERROR, G_IO_ERROR_CANCELLED, "script cancelled"));
-		load_data_unref (load_data);
 		return;
 	}
 
-	pixbuf = NULL;
-	if (status == 0)
-		pixbuf = gnome_desktop_thumbnail_factory_load_from_tempfile (self->priv->thumb_factory,
-									     &load_data->thumbnailer_tmpfile);
+	if (status == 0) {
+		GdkPixbuf *pixbuf = gnome_desktop_thumbnail_factory_load_from_tempfile (
+			self->priv->thumb_factory,
+			&load_data->thumbnailer_tmpfile);
 
-	if (pixbuf != NULL) {
-		cairo_surface_t *surface;
+		if (pixbuf != NULL) {
+			cairo_surface_t *surface;
 
-		surface = _cairo_image_surface_create_from_pixbuf (pixbuf);
-		original_image_loaded_correctly (self, load_data, surface, 0, 0);
+			surface = _cairo_image_surface_create_from_pixbuf (pixbuf);
+			original_image_loaded_correctly (self, load_data, surface, 0, 0);
 
-		cairo_surface_destroy (surface);
-		g_object_unref (pixbuf);
+			cairo_surface_destroy (surface);
+			g_object_unref (pixbuf);
+			return;
+		}
 	}
-	else
-		failed_to_load_original_image (self, load_data);
 
-	load_data_unref (load_data);
+	failed_to_load_original_image (self, load_data);
 }
 
 
@@ -772,29 +820,44 @@ load_with_system_thumbnailer (GthThumbLoader *self,
 	GError *error = NULL;
 
 	uri = g_file_get_uri (load_data->file_data->file);
-	if (gnome_desktop_thumbnail_factory_generate_from_script (self->priv->thumb_factory,
-								  uri,
-								  gth_file_data_get_mime_type (load_data->file_data),
-								  &load_data->thumbnailer_pid,
-								  &load_data->thumbnailer_tmpfile,
-								  &error))
+	if (gnome_desktop_thumbnail_factory_generate_from_script (
+		    self->priv->thumb_factory,
+		    uri,
+		    gth_file_data_get_mime_type (load_data->file_data),
+		    &load_data->thumbnailer_pid,
+		    &load_data->thumbnailer_tmpfile,
+		    &error))
 	{
-		load_data->thumbnailer_watch = g_child_watch_add (load_data->thumbnailer_pid,
-								  watch_thumbnailer_cb,
-								  load_data);
-		load_data->thumbnailer_timeout = g_timeout_add (MAX_THUMBNAILER_LIFETIME,
-								kill_thumbnailer_cb,
-								load_data);
-		load_data->cancellable_watch = g_timeout_add (CHECK_CANCELLABLE_DELAY,
-							      check_cancellable_cb,
-							      load_data);
+		load_data_ref (load_data);
+		load_data->thumbnailer_watch = g_child_watch_add_full (
+			G_PRIORITY_DEFAULT_IDLE,
+			load_data->thumbnailer_pid,
+			watch_thumbnailer_cb,
+			load_data,
+			(GDestroyNotify) load_data_unref);
+
+		load_data_ref (load_data);
+		load_data->thumbnailer_timeout = g_timeout_add_full (
+			G_PRIORITY_DEFAULT,
+			MAX_THUMBNAILER_LIFETIME,
+			kill_thumbnailer_cb,
+			load_data,
+			(GDestroyNotify) load_data_unref);
+
+		load_data_ref (load_data);
+		load_data->cancellable_watch = g_timeout_add_full (
+			G_PRIORITY_DEFAULT,
+			CHECK_CANCELLABLE_DELAY,
+			check_cancellable_cb,
+			load_data,
+			(GDestroyNotify) load_data_unref);
 	}
 	else {
 		g_clear_error (&error);
 		failed_to_load_original_image (self, load_data);
-		load_data_unref (load_data);
 	}
 
+	load_data_unref (load_data);
 	g_free (uri);
 }
 
