@@ -9,10 +9,10 @@ if len(sys.argv) <= 3:
 
 infile = sys.argv[1]
 outfile = sys.argv[2]
-headers = "\n".join(map(lambda x: "#include <gthumb/{0}>".format(x), sys.argv[3:]))
+headers = "\n".join(map(lambda x: "#include <{0}>".format(x), sys.argv[3:]))
 
 with open(outfile, 'w') as to_file, open(infile) as from_file:
     for line in iter(from_file.readline, ''):
         if line == "@HEADERS@\n":
             line = headers + "\n"
-        to_file.write(line) 
+        to_file.write(line)
